# 导入requests 库
import requests
# 导入 tqdm
from tqdm import tqdm


def download(url: str, file_name: str):
    '''
    根据文件直链和文件名下载文件

    Parameters
    ----------
    url: 文件直链
    file_name : 文件名（文件路径）

    '''
    # 文件下载直链
    # 请求头
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.132 Safari/537.36 QIHU 360SE'
    }

    # 发起 head 请求，即只会获取响应头部信息
    head = requests.head(url, headers=headers)
    # 文件大小，以 B 为单位
    file_size = head.headers.get('Content-Length')
    if file_size is not None:
        file_size = int(file_size)
    response = requests.get(url, headers=headers, stream=True)
    # 一块文件的大小
    chunk_size = 1024
    bar = tqdm(total=file_size, desc=f'下载文件 {file_name}')
    with open(file_name, mode='wb') as f:
        # 写入分块文件
        for chunk in response.iter_content(chunk_size=chunk_size):
            f.write(chunk)
            bar.update(chunk_size)
    # 关闭进度条
    bar.close()


if "__main__" == __name__:
    url = 'https://github.com/github/gitignore/archive/refs/heads/master.zip'
    file_name = 'gitignore-master.zip'
    download(url, file_name)



import requests
# 待下载部分的文件起始位置
start = 0
# 待下载部分的文件终止位置
end = 1000
# 每次读取的大小
chunk_size = 128
# 记录下载的位置
seek = start
# 请求头
headers = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.132 Safari/537.36 QIHU 360SE'
}
# 这是核心！设定下载的范围
headers['Range'] = f'bytes={start}-{end}'
# 下载链接
url = 'https://github.com/github/gitignore/archive/refs/heads/master.zip'
file_name = 'gitignore-master.zip'
response = requests.get(url, headers=headers, stream=True)

for chunk in response.iter_content(chunk_size=chunk_size):
    _seek = min(seek+chunk_size, end)
    print(f'下载: {seek}-{_seek}')
    seek = _seek



# 总大小
from __future__ import annotations
# total = 121
# # 每一块的大小
# step = 20
# # 分多块
# parts = [(start, min(start+step, total)) for start in range(0, total, step)]

# print(parts)

# # # 看最后那一部分的最后一个数字是否等于总大小，不等于就新加一块
# if parts[-1][-1] != total:
#     start = step*len(parts)
#     end = total
#     parts.append((start, total))
# print(parts)


def split(start: int, end: int, step: int) -> list[tuple[int, int]]:
    '''
    将指定区间的数切割为多个区间

    Parameters
    ----------
    start :起始位置
    end : 终止位置
    step : 区间长度

    Return
    ------
    区间元组构成的列表
    '''
    # 分多块
    parts = [(start, min(start+step, end))
             for start in range(0, end, step)]
    # 看最后那一部分的最后一个数字是否等于总大小，不等于就新加一块
    if parts[-1][-1] != end:
        start = step*len(parts)
        parts.append((start, end))
    return parts


if "__main__" == __name__:
    # 起始位置
    start = 1
    # 终止位置
    total = 102
    # 区间长度
    step = 20
    parts = split(start, total, step)
    print(parts)



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
'''
Created on   :2021/06/03 00:46:04
@author      :Caihao (Chris) Cui
@file        :app.py
@content     :xxx xxx xxx
@version     :0.1.0 in DevOpt
@License :   (C)Copyright 2021 MIT
'''

# here put the import lib

from src.pySpeedDownloader.pydownloader import run

import os
import json
from gooey import Gooey, GooeyParser
import logging

logging.basicConfig(
    level=logging.INFO,
    # filename='output.log',
    filemode='w',
    datefmt='%Y/%m/%d %H:%M:%S',
    format='%(asctime)s-%(levelname)s - %(message)s')
# format='%(levelname)s-%(message)s')

logger = logging.getLogger(__name__)

W, H = (480, 540)


@Gooey(
    program_name="Py-Downloader APP",
    default_size=(W, H),
    advanced=True,
    required_cols=1,
    optional_cols=2,
    progress_regex=r"(\d+)%",
    tabbed_groups=True,
    # dump_build_config=True,   # Dump the JSON Gooey uses to configure itself
    # load_build_config=None,    # Loads a JSON Gooey-generated configuration
    # navigation="Tabbed",
    menu=[
        {
            "name":
            "File",
            "items": [
                {
                    "type": "AboutDialog",
                    "menuTitle": "About",
                    "name": "Py-Downloader APP",
                    "description": "Py-Downloader APP",
                    "version": "0.5.0",
                    "copyright": "2021",
                    "developer": "Chris.Cui"
                },
                {
                    "type":
                    "MessageDialog",
                    "menuTitle":
                    "Information",
                    "caption":
                    "Py-Downloader APP",
                    "message":
                    "This repo is built by Chris Cui.",
                },
            ],
        },
        {
            "name":
            "Help",
            "items": [{
                "type":
                "Link",
                "menuTitle":
                "Documentation",
                "url":
                "https://github.com/cuicaihao/py-downloader-app"
            }],
        },
    ],
)
def parse_args():
    """
    Use GooeyParser to build up the arguments we will use in our script
    Save the arguments in a default json file so that we can retrieve them
    every time we run the script.
    """
    # stored_args = {}
    # get the script name without the extension & use it to build up
    # the json filename
    script_name = os.path.splitext(os.path.basename(__file__))[0]
    args_file = "{}-args.json".format(script_name)
    # Read in the prior arguments as a dictionary
    if os.path.isfile(args_file):
        with open(args_file) as data_file:
            stored_args = json.load(data_file)
        # return stored_args

    settings_msg = "Py-Downloader APP"
    parser = GooeyParser(description=settings_msg)

    parser.add_argument('--version', '-v', action='version',
                        version='%(prog)s 0.5.0')

    parser.add_argument('url', type=str,  action="store", default="https://github.com/github/gitignore/archive/refs/heads/master.zip",
                        help="Download URL")
    parser.add_argument('file_name', type=str, action="store", default="gitignore-master.zip",
                        help="Name of the download file.")
    parser.add_argument(
        'output_dir', type=str,  action="store", default="./download", help="Download directory", widget="DirChooser",)

    args = parser.parse_args()
    # Store the values of the arguments so we have them next time we run
    with open(args_file, "w") as data_file:
        # Using vars(args) returns the data as a dictionary
        json.dump(vars(args), data_file)
    return args


if __name__ == "__main__":
    conf = parse_args()
    # Output Input Configurations
    for arg in vars(conf):
        logging.info("{}:{}".format(arg, getattr(conf, arg)))
    file_md5 = run(conf.url, conf.file_name, conf.output_dir, check=False)
    logging.info(f"{conf.file_name} MD5: {file_md5}")
    print("\r" * 3)



# 导入requests 库
import requests
# 文件下载直链
url = 'https://github.com/github/gitignore/archive/refs/heads/master.zip'
file_name = 'gitignore-master.zip'
# 请求头
headers = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.132 Safari/537.36 QIHU 360SE'
}
print('正在下载文件......')
# 发起请求
response = requests.get(url, headers=headers)

content = response.content

# 以 wb 的模式打开文件
with open(file_name, mode='wb') as f:
    # 写入响应内容
    f.write(content)
print(f'文件下载成功！文件名 {file_name}')



import setuptools
from pathlib import Path
# with open("ReadMe.md", "r", encoding="utf-8") as fh:
#     long_description = fh.read()

setuptools.setup(
    name="pySpeedDownloader",
    version="0.1.1",
    author="Chris Cui",
    author_email="",
    description="A powerful python based downloader module.",
    long_description=Path("ProjectDescription.md").read_text(),
    long_description_content_type="text/markdown",
    url="https://github.com/cuicaihao/py-downloader-app",
    package_dir={"": "src"},
    project_urls={
    },
    packages=setuptools.find_packages(
        where="src", exclude=['test', 'target', 'build', 'main']),
    python_requires=">=3.7",
    install_requires=[
        'tqdm>=4.40.0',
        'requests>=2.9.1',
        'multitasking>=0.0.8',
        'signal',
        'retry>=0.9.1'
    ],

)

# python setup.py sdist bdist_wheel
# twine upload dist/*



from __future__ import annotations
import hashlib
from pathlib import Path
import argparse
# 用于显示进度条
from tqdm import tqdm
# 用于发起网络请求
import requests
# 用于多线程操作
import multitasking
import signal
# 导入 retry 库以方便进行下载出错重试
from retry import retry
signal.signal(signal.SIGINT, multitasking.killall)

# 请求头
headers = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.132 Safari/537.36 QIHU 360SE'
}
# 定义 1 MB 多少为 B
MB = 1024**2


def split(start: int, end: int, step: int) -> list[tuple[int, int]]:
    # 分多块
    parts = [(start, min(start+step, end))
             for start in range(0, end, step)]
    # 看最后那一部分的最后一个数字是否等于总大小，不等于就新加一块
    if parts[-1][-1] != end:
        start = step*len(parts)
        parts.append((start, end))
    return parts


def get_file_size(url: str, raise_error: bool = False) -> int:
    '''
    获取文件大小

    Parameters
    ----------
    url : 文件直链
    raise_error : 如果无法获取文件大小，是否引发错误

    Return
    ------
    文件大小（B为单位）
    如果不支持则会报错

    '''
    response = requests.head(url)
    file_size = response.headers.get('Content-Length')
    if file_size is None:
        if raise_error is True:
            raise ValueError('该文件不支持多线程分段下载！')
        return file_size
    return int(file_size)


def hd5check(file_name):
    # with open(file_name) as file_to_check:
    #     # read contents of the file
    #     data = file_to_check.read()
    #     # pipe contents of the file through
    #     md5_returned = hashlib.md5(data).hexdigest()
    return hashlib.md5(open(file_name, 'rb').read()).hexdigest()


def ask_user(Question):
    check = str(input(f"{Question}? (Y/N): ")).lower().strip()
    try:
        if check[0] == 'y':
            return True
        elif check[0] == 'n':
            return False
        else:
            return False  # do not overwrite
            # print('Invalid Input')
            # return ask_user(Question)
    except Exception as error:
        print("Please enter valid inputs")
        print(error)
        return ask_user(Question)


def pre_download_check(file_name, output_dir):
    if not Path(output_dir).is_dir():
        print(f"{output_dir} does not exist!")
        Path(output_dir).mkdir(parents=True, exist_ok=True)

    download_file_path = Path(output_dir) / file_name
    if download_file_path.exists():
        return True
    else:
        return False


def download(url: str, file_name: str, output_dir: str = './download', retry_times: int = 3, each_size=16*MB) -> None:
    '''
    根据文件直链和文件名下载文件

    Parameters
    ----------
    url : 文件直链
    file_name : 文件名
    retry_times: 可选的，每次连接失败重试次数
    Return
    ------
    None

    '''
    download_file_path = Path(output_dir) / file_name
    if pre_download_check(file_name, output_dir):
        answer_yes = ask_user(
            f" ⛔️ {str(download_file_path)} already exists, overwrite it?")
    else:
        answer_yes = True

    if answer_yes:
        download_file_path = str(download_file_path)
        f = open(download_file_path, 'wb')
        file_size = get_file_size(url)

        @retry(tries=retry_times)
        @multitasking.task
        def start_download(start: int, end: int) -> None:
            '''
            根据文件起止位置下载文件

            Parameters
            ----------
            start : 开始位置
            end : 结束位置
            '''
            _headers = headers.copy()
            # 分段下载的核心
            _headers['Range'] = f'bytes={start}-{end}'
            # 发起请求并获取响应（流式）
            response = session.get(url, headers=_headers, stream=True)
            # 每次读取的流式响应大小
            chunk_size = 128
            # 暂存已获取的响应，后续循环写入
            chunks = []
            for chunk in response.iter_content(chunk_size=chunk_size):
                # 暂存获取的响应
                chunks.append(chunk)
                # 更新进度条
                bar.update(chunk_size)
            f.seek(start)
            for chunk in chunks:
                f.write(chunk)
            # 释放已写入的资源
            del chunks

        session = requests.Session()
        # 分块文件如果比文件大，就取文件大小为分块大小
        each_size = min(each_size, file_size)

        # 分块
        parts = split(0, file_size, each_size)
        print(f'Parts Number: {len(parts)}')
        # 创建进度条
        bar = tqdm(total=file_size, desc=f'Downloading: {file_name}')
        for part in parts:
            start, end = part
            start_download(start, end)
        # 等待全部线程结束
        multitasking.wait_for_tasks()
        f.close()
        bar.close()

    return hd5check(download_file_path)


def test_download_from_github():
    url = 'https://github.com/github/gitignore/archive/refs/heads/master.zip'
    file_name = 'gitignore-master.zip'
    file_md5 = download(url, file_name, )
    print(file_md5)


def run(url, file_name, output_dir):
    print(f"Downloading {file_name}...")
    file_md5 = download(url, file_name, output_dir)
    return file_md5


if "__main__" == __name__:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--url', type=str, default="https://github.com/github/gitignore/archive/refs/heads/master.zip", help='foo help')
    parser.add_argument('--file_name', type=str,
                        default="gitignore-master.zip", help='foo help')
    parser.add_argument('--output_dir', type=str,
                        default="./download", help='foo help')
    args = parser.parse_args()
    # 开始下载文件
    file_md5 = run(args.url, args.file_name, args.output_dir)
    print(f"File MD5 Check: {file_md5}")



import time
# 导入用于多线程操作的库
# 这样子仅需要在自定义的函数前面使用装饰器即可将函数开启新的线程
import multitasking
import signal
# 按快捷键 ctrl + c 终止已开启的全部线程
signal.signal(signal.SIGINT, multitasking.killall)

# 多线程装饰器


@multitasking.task
def say(number: int):
    print(number)
    time.sleep(0.5)


start_time = time.time()
for i in range(5):
    say(i)
# 等待全部线程执行完毕
multitasking.wait_for_tasks()
end_time = time.time()
print('耗时:', end_time-start_time, '秒')



from src.pySpeedDownloader import pydownloader

url = "https://github.com/tensorflow/tensorflow/archive/refs/heads/master.zip"
file_name = "tensorflow-master.zip"
output_dir = "download/tf"

file_md5 = pydownloader.run(url, file_name, output_dir)
print(file_md5)

url = "https://github.com/github/gitignore/archive/refs/heads/master.zip"
file_name = "gitignore-master.zip"

output_dir = "download/gitignore"

file_md5 = pydownloader.run(url, file_name, output_dir)
print(file_md5)



# 导入requests 库
import requests
# 导入 tqdm
from tqdm import tqdm
# 文件下载直链
url = 'https://github.com/github/gitignore/archive/refs/heads/master.zip'
file_name = 'gitignore-master.zip'
# 请求头
headers = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.132 Safari/537.36 QIHU 360SE'
}

# 发起 head 请求，即只会获取响应头部信息
head = requests.head(url, headers=headers)
# 文件大小，以 B 为单位
file_size = head.headers.get('Content-Length')
if file_size is not None:
    file_size = int(file_size)
response = requests.get(url, headers=headers, stream=True)
# 一块文件的大小
chunk_size = 1024
bar = tqdm(total=file_size, desc=f'下载文件 {file_name}')
with open(file_name, mode='wb') as f:
    # 写入分块文件
    for chunk in response.iter_content(chunk_size=chunk_size):
        f.write(chunk)
        bar.update(chunk_size)
# 关闭进度条
bar.close()
